@@ -29,6 +29,7 @@ from .constants import (
     CHERRIES,
     CONTINUES,
     CURRENT_LEVEL,
+    CURRENT_PAGE_POSITION,
     DOOR_TRANSITION_TIMER,
     ENEMIES_DEFEATED,
     ENEMY_HEALTH,
@@ -65,12 +66,14 @@ from .constants import (
     PLAYER_Y_POSITION,
     SCREEN_HEIGHT,
     SCREEN_WIDTH,
+    SCROLL_DIRECTION,
     STARMAN_TIMER,
     STOPWATCH_TIMER,
     SUB_AREA,
     SUBSPACE_COINS,
     SUBSPACE_STATUS,
     SUBSPACE_TIMER,
+    TOTAL_PAGES_IN_SUB_AREA,
     VEGETABLES_PULLED,
     WORLD_NUMBER,
     Y_POSITION_WRAPAROUND_THRESHOLD,
@@ -90,8 +93,6 @@ class SuperMarioBros2Env(gym.Env):
     based on the RAM values available in the info dict.
     """
 
-    metadata = {'render_modes': ['human'], 'render_fps': 60}
-
     def __init__(
         self,
         init_config: InitConfig,
@@ -99,8 +100,8 @@ class SuperMarioBros2Env(gym.Env):
         max_episode_steps: Optional[int] = None,
         action_type: ActionType = "simple",
         reset_on_life_loss: bool = False,
-        use_save_state: bool = True,
         render_fps: Optional[int] = None,
+        frame_method: str = "rgb",
     ):
         """Initialize the SMB2 environment.
 
@@ -110,18 +111,27 @@ class SuperMarioBros2Env(gym.Env):
             max_episode_steps: Maximum steps per episode (for truncation)
             action_type: Type of action space
             reset_on_life_loss: If True, episode terminates when Mario loses a life
-            use_save_state: If False, start from beginning without loading save state
             render_fps: FPS for human rendering (None = no limit, good for training)
+            frame_method: Frame rendering method ('rgb', 'grayscale')
+                - 'rgb': RGB rendering
+                - 'grayscale': Grayscale rendering (faster, 67% less memory)
         """
         super().__init__()
 
         self.render_mode = render_mode
         self.max_episode_steps = max_episode_steps
         self.reset_on_life_loss = reset_on_life_loss
-        self.use_save_state = use_save_state
         self.init_config = init_config
         self.render_fps = render_fps
 
+        # Validate and store frame method
+        valid_frame_methods = ["rgb", "grayscale"]
+        if frame_method not in valid_frame_methods:
+            raise ValueError(
+                f"Invalid frame_method '{frame_method}'. Must be one of {valid_frame_methods}"
+            )
+        self.frame_method = frame_method
+
         # Store relevant attributes (only meaningful for built-in ROM mode)
         if not self.init_config.rom_path:  # Built-in ROM mode
             self.starting_level = self.init_config.level
@@ -154,8 +164,8 @@ class SuperMarioBros2Env(gym.Env):
         if not os.path.exists(rom_path):
             raise FileNotFoundError(f"ROM file not found: {rom_path}")
 
-        # Initialize TetaNES
-        self._nes = NesEnv(headless=False)
+        # Initialize TetaNES with frame rendering method
+        self._nes = NesEnv(headless=False, frame_method=self.frame_method)
 
         # Load ROM
         with open(rom_path, 'rb') as f:
@@ -165,10 +175,21 @@ class SuperMarioBros2Env(gym.Env):
 
     def _init_spaces(self) -> None:
         """Initialize observation and action spaces."""
-        # Define observation space
-        self.observation_space = spaces.Box(
-            low=0, high=255, shape=(SCREEN_HEIGHT, SCREEN_WIDTH, 3), dtype=np.uint8
-        )
+        # Define observation space based on frame method
+        if self.frame_method == "grayscale":
+            self.observation_space = spaces.Box(
+                low=0,
+                high=255,
+                shape=(SCREEN_HEIGHT, SCREEN_WIDTH),
+                dtype=np.uint8,
+            )
+        else:  # rgb
+            self.observation_space = spaces.Box(
+                low=0,
+                high=255,
+                shape=(SCREEN_HEIGHT, SCREEN_WIDTH, 3),
+                dtype=np.uint8,
+            )
 
         # Define action space based on action_type
         if self.action_type == "all":
@@ -242,16 +263,15 @@ class SuperMarioBros2Env(gym.Env):
         self._done = False
         self._episode_steps = 0
 
-        if self.use_save_state:
-            save_path = self.init_config.get_save_state_path()
+        save_path = self.init_config.get_save_state_path()
 
-            if save_path and not os.path.exists(save_path):
-                raise FileNotFoundError(f"Save state file not found: {save_path}")
+        if save_path and not os.path.exists(save_path):
+            raise FileNotFoundError(f"Save state file not found: {save_path}")
 
-            if save_path:
-                self.load_state_from_path(save_path)
+        if save_path:
+            self.load_state_from_path(save_path)
         else:
-            # When not using save state, navigate to character selection screen
+            # When no save state, navigate to character selection screen
             # Wait for title screen to appear
             for _ in range(120):  # 2 seconds
                 self._nes.step([False] * 8, render=False)
@@ -274,10 +294,11 @@ class SuperMarioBros2Env(gym.Env):
 
         info = self.info
 
-        # Initialize life tracking for detecting life loss
+        # Initialize tracking for detecting life loss and level completion
         self._previous_lives = self.lives
+        self._previous_levels_finished = self.levels_finished.copy()
         if self.render_mode == 'human':
-            self.render()
+            self.render(obs)
 
         return np.array(obs), info
 
@@ -313,11 +334,13 @@ class SuperMarioBros2Env(gym.Env):
         if life_lost:
             info['life_lost'] = True
 
-        # Update life tracking for next step
+        # Update tracking for next step
+        level_completed = self.level_completed
         self._previous_lives = self.lives
+        self._previous_levels_finished = self.levels_finished.copy()
 
         # 4. Check termination
-        terminated = self.is_game_over or life_lost
+        terminated = self.is_game_over or life_lost or level_completed
         truncated = (
             self.max_episode_steps is not None and self._episode_steps >= self.max_episode_steps
         )
@@ -327,13 +350,16 @@ class SuperMarioBros2Env(gym.Env):
 
         # Render if in human mode
         if self.render_mode == 'human':
-            self.render()
+            self.render(obs)
 
         return np.array(obs), reward, terminated, truncated, info
 
-    def render(self) -> Optional[np.ndarray]:
+    def render(self, obs: np.ndarray) -> Optional[np.ndarray]:
         """Render the environment.
 
+        Args:
+            obs: Observation array to render.
+
         Returns:
             RGB array for display, None if no render mode
         """
@@ -342,7 +368,6 @@ class SuperMarioBros2Env(gym.Env):
             if not self._pygame_initialized:
                 self._init_rendering()
 
-            obs = self._nes.get_observation()
             if self._screen is not None:
                 import pygame
 
@@ -399,6 +424,9 @@ class SuperMarioBros2Env(gym.Env):
             'area': self.area,
             'sub_area': self.sub_area,
             'spawn_page': self.spawn_page,
+            'current_page_position': self.current_page_position,
+            'total_pages_in_sub_area': self.total_pages_in_sub_area,
+            'is_vertical_area': self.is_vertical_area,
             'global_coordinates': self.global_coordinate_system,
             'character': self.character,
             'hearts': self.hearts,
@@ -417,6 +445,7 @@ class SuperMarioBros2Env(gym.Env):
             'vegetables_pulled': self.vegetables_pulled,
             'subspace_status': self.subspace_status,
             'level_transition': self.level_transition,
+            'level_completed': self.level_completed,
             'door_transition_timer': self.door_transition_timer,
             'enemies_defeated': self.enemies_defeated,
         }
@@ -533,6 +562,24 @@ class SuperMarioBros2Env(gym.Env):
         page = self._read_ram_safe(PAGE, default=0)
         return page
 
+    @property
+    def current_page_position(self) -> int:
+        """Get current page position in sub-area."""
+        page_pos = self._read_ram_safe(CURRENT_PAGE_POSITION, default=0)
+        return page_pos
+
+    @property
+    def total_pages_in_sub_area(self) -> int:
+        """Get total number of pages in the current sub-area."""
+        total_pages = self._read_ram_safe(TOTAL_PAGES_IN_SUB_AREA, default=0)
+        return total_pages
+
+    @property
+    def is_vertical_area(self) -> bool:
+        """Check if current area has vertical scrolling."""
+        direction = self._read_ram_safe(SCROLL_DIRECTION, default=0)
+        return direction == 1
+
     @property
     def global_coordinate_system(self) -> GlobalCoordinate:
         """Get global coordinate system combining level structure with player position.
@@ -683,16 +730,16 @@ class SuperMarioBros2Env(gym.Env):
         """Get level transition state.
 
         NOTE: This value at 0x04EC appears to change for less than a frame.
-        The game sets it to non-zero and immediately clears it back to 0 
-        within the same frame's CPU execution (as seen in disassembly at 
+        The game sets it to non-zero and immediately clears it back to 0
+        within the same frame's CPU execution (as seen in disassembly at
         $E66D: STA $04EC). Therefore, we cannot reliably detect transitions
         by polling this value once per frame.
-        For reliable level completion detection, use the increase in 
+        For reliable level completion detection, use the increase in
         'levels_finished' counter instead.
 
         Values (theoretical):
         0 - normal gameplay
-        1 - restart same level  
+        1 - restart same level
         2 - game over
         3 - end level, go to bonus game (level completed)
         4 - warp
@@ -700,6 +747,24 @@ class SuperMarioBros2Env(gym.Env):
         # This will almost always return 0 due to sub-frame clearing
         return self._read_ram_safe(LEVEL_TRANSITION, default=0)
 
+    @property
+    def level_completed(self) -> bool:
+        """Detect if a level was just completed.
+
+        Returns True if any character's levels_finished counter has increased
+        since the last step.
+        """
+        if not hasattr(self, '_previous_levels_finished'):
+            return False
+
+        current_levels_finished = self.levels_finished
+        for char_name in ['mario', 'peach', 'toad', 'luigi']:
+            if current_levels_finished[char_name] > self._previous_levels_finished.get(
+                char_name, 0
+            ):
+                return True
+        return False
+
     # ---- Validators ------------------------------------------------
 
     def _validate_and_convert_action(self, action: int) -> np.ndarray:
@@ -777,6 +842,28 @@ class SuperMarioBros2Env(gym.Env):
             raise FileNotFoundError(f"Save state file not found: {filepath}")
         self._nes.load_state_from_path(filepath)
 
+    def set_frame_speed(self, speed: float) -> None:
+        """Set the frame speed for faster/slower emulation.
+
+        Args:
+            speed: Frame speed multiplier (1.0 = normal speed, 2.0 = 2x speed, etc.)
+                   Must be positive.
+
+        Raises:
+            ValueError: If speed is not positive
+        """
+        if speed <= 0.0:
+            raise ValueError("Frame speed must be positive")
+        self._nes.set_frame_speed(speed)
+
+    def get_frame_speed(self) -> float:
+        """Get the current frame speed multiplier.
+
+        Returns:
+            Current frame speed (1.0 = normal speed)
+        """
+        return self._nes.get_frame_speed()
+
     def close(self) -> None:
         """Close the environment and clean up resources."""
         if hasattr(self, '_pygame_initialized') and self._pygame_initialized:
